package chart_info

import (
	"hongze/hongze_yb/models/request"
	responseModel "hongze/hongze_yb/models/response"
	"hongze/hongze_yb/models/tables/chart_edb_mapping"
	"hongze/hongze_yb/models/tables/chart_info"
	"time"
)

type ChartInfoDetailResp struct {
	ChartInfo            *chart_info.ChartInfoView
	EdbInfoList          []*chart_edb_mapping.ChartEdbInfoMappingList
	XEdbIdValue          []int   `description:"柱方图的x轴数据，指标id"`
	XDataList            []XData `description:"商品价格曲线的X轴数据"`
	YDataList            []YData `description:"柱方图的y轴数据"`
	MyChartInfo          *responseModel.MyChartItem
	CorrelationChartInfo *CorrelationInfo `description:"相关性图表信息"`
	DataResp             interface{}      `description:"图表数据,根据图的类型而定的,没有确定的数据格式"`
}

type ChartLibChartInfoDetailResp struct {
	ChartInfo            *chart_info.ChartInfoView
	EdbInfoList          []*chart_edb_mapping.ChartEdbInfoMappingTimeString
	XEdbIdValue          []int            `description:"柱方图的x轴数据，指标id"`
	XDataList            []XData          `description:"商品价格曲线的X轴数据"`
	YDataList            []YData          `description:"柱方图的y轴数据"`
	CorrelationChartInfo *CorrelationInfo `description:"相关性图表信息"`
	DataResp             interface{}      `description:"图表数据,根据图的类型而定的,没有确定的数据格式"`
	WaterMark            string           `description:"水印"`
}

// XData 商品价格曲线的的x轴数据
type XData struct {
	Name   string `description:"别名"`
	NameEn string `description:"英文别名"`
}

// YData 柱方图的y轴数据
type YData struct {
	Date           string          `description:"数据日期"`
	ConfigDate     time.Time       `description:"配置的日期" json:"-"`
	Color          string          `description:"数据颜色"`
	Name           string          `description:"别名"`
	NameEn         string          `description:"英文别名"`
	Value          []float64       `description:"每个指标的值"`
	NoDataEdbList  []int           `description:"没有数据的指标列表"`
	XEdbInfoIdList []int           `description:"对应X轴的指标id列表"`
	EdbValMap      map[int]float64 `description:"指标与值的对应" json:"-"`
	M              []int           `description:"对应开始日期的间隔值" json:"-"`
	NameList       []string        `description:"每个值对应的名称"`
	EnNameList     []string        `description:"每个值对应的英文名称"`
	SeriesEdb      struct {
		SeriesId  int `description:"因子指标系列ID"`
		EdbInfoId int `description:"指标ID"`
	} `description:"对应的系列指标"`
}

type CorrelationInfo struct {
	LeadValue       int    `description:"领先值"`
	LeadUnit        string `description:"领先单位"`
	StartDate       string `description:"开始日期"`
	EndDate         string `description:"结束日期"`
	EdbInfoIdFirst  int    `description:"A指标ID"`
	EdbInfoIdSecond int    `description:"B指标ID"`
	PeriodData      string `description:"X轴-期数数据"`
	CorrelationData string `description:"Y轴-相关性系数"`
	AnalysisMode    int    `description:"分析模式: 0-单因子; 1-多因子"`
}

// RollingCorrelationChartDataResp 滚动相关性图
type RollingCorrelationChartDataResp struct {
	XDateTimeValue []string `description:"滚动相关性图的x轴数据，日期数据"`
	YDataList      []YData  `description:"滚动相关性图的y轴数据"`
}

// ProfitFutureGoodChartResp 商品利润图
type ProfitFutureGoodChartResp struct {
	XDataList    []XData
	YDataList    []YData
	ProfitName   string               `description:"利润的名称"`
	ProfitNameEn string               `description:"利润的英文名称"`
	Extra        request.ChartInfoReq `description:"柱方图的配置"`
}

// RadarChartInfoResp 雷达图数据
type RadarChartInfoResp struct {
	YDataList   []RadarYData `description:"数据列"`
	XEdbIdValue []int
}

// RadarYData 雷达图的y轴数据
type RadarYData struct {
	Date  string    `description:"数据日期"`
	Color string    `description:"数据颜色"`
	Name  string    `description:"别名"`
	Value []float64 `description:"每个指标的值"`
}
