package fe_calendar_matter

import "time"

// FeCalendarMatter 外汇日历-事项表
type FeCalendarMatter struct {
	FeCalendarMatterId  int       `gorm:"primaryKey;column:fe_calendar_matter_id;type:int(10) unsigned;not null" json:"-"`                                            // 事项Id
	ChartPermissionId   int       `gorm:"index:idx_chart_permission_id;column:chart_permission_id;type:int(10) unsigned;not null;default:0" json:"chartPermissionId"` // 品种Id
	ChartPermissionName string    `gorm:"column:chart_permission_name;type:varchar(128);not null;default:''" json:"chartPermissionName"`                              // 品种名称
	MatterMonth         string    `gorm:"index:idx_matter_month;column:matter_month;type:varchar(16);not null;default:''" json:"matterMonth"`                         // 事项年月：格式2006-01
	MatterDate          time.Time `gorm:"index:idx_matter_date;column:matter_date;type:date" json:"matterDate"`                                                       // 事项日期
	Title               string    `gorm:"column:title;type:varchar(255);not null;default:''" json:"title"`                                                            // 标题
	MatterType          int       `gorm:"column:matter_type;type:tinyint(4) unsigned;not null;default:0" json:"matterType"`                                           // 事项类型：1-自定义事项；2-基础指标；3-预测指标
	EdbInfoId           int       `gorm:"column:edb_info_id;type:int(10) unsigned;not null;default:0" json:"edbInfoId"`                                               // 指标Id
	EdbUniqueCode       string    `gorm:"column:edb_unique_code;type:varchar(128);not null;default:''" json:"edbUniqueCode"`                                          // 指标唯一编码
	EdbCode             string    `gorm:"column:edb_code;type:varchar(128);not null;default:''" json:"edbCode"`                                                       // 指标编码
	FontColor           string    `gorm:"column:font_color;type:varchar(64);not null;default:''" json:"fontColor"`                                                    // 字体颜色
	FillingColor        string    `gorm:"column:filling_color;type:varchar(64);not null;default:''" json:"fillingColor"`                                              // 填充颜色
	FontBold            int       `gorm:"column:font_bold;type:tinyint(4) unsigned;not null;default:0" json:"fontBold"`                                               // 字体加粗：0-否；1-是
	Sort                int       `gorm:"column:sort;type:int(10) unsigned;not null;default:0" json:"sort"`                                                           // 排序
	SysUserId           int       `gorm:"column:sys_user_id;type:int(10) unsigned;not null;default:0" json:"sysUserId"`                                               // 创建人Id
	SysUserName         string    `gorm:"column:sys_user_name;type:varchar(128);not null;default:''" json:"sysUserName"`                                              // 创建人姓名
	CreateTime          time.Time `gorm:"column:create_time;type:datetime" json:"createTime"`                                                                         // 创建时间
	ModifyTime          time.Time `gorm:"column:modify_time;type:datetime" json:"modifyTime"`                                                                         // 更新时间
}

// TableName get sql table name.获取数据库表名
func (m *FeCalendarMatter) TableName() string {
	return "fe_calendar_matter"
}

// FeCalendarMatterColumns get sql column name.获取数据库列名
var FeCalendarMatterColumns = struct {
	FeCalendarMatterId  string
	ChartPermissionId   string
	ChartPermissionName string
	MatterMonth         string
	MatterDate          string
	Title               string
	MatterType          string
	EdbInfoId           string
	EdbUniqueCode       string
	EdbCode             string
	FontColor           string
	FillingColor        string
	FontBold            string
	Sort                string
	SysUserId           string
	SysUserName         string
	CreateTime          string
	ModifyTime          string
}{
	FeCalendarMatterId:  "fe_calendar_matter_id",
	ChartPermissionId:   "chart_permission_id",
	ChartPermissionName: "chart_permission_name",
	MatterMonth:         "matter_month",
	MatterDate:          "matter_date",
	Title:               "title",
	MatterType:          "matter_type",
	EdbInfoId:           "edb_info_id",
	EdbUniqueCode:       "edb_unique_code",
	EdbCode:             "edb_code",
	FontColor:           "font_color",
	FillingColor:        "filling_color",
	FontBold:            "font_bold",
	Sort:                "sort",
	SysUserId:           "sys_user_id",
	SysUserName:         "sys_user_name",
	CreateTime:          "create_time",
	ModifyTime:          "modify_time",
}
